Module errors

Module errors 

Error types that are reported by various tensor operations.

Structs

DimensionError
Error in a tensor operation if the dimension count is incorrect.

Enums

ExpandError
Errors that can occur while expanding a tensor.
FromDataError
Errors that can occur when constructing a tensor from existing data.
ReshapeError
Errors that can occur while reshaping a layout or tensor.
SliceError
Errors that can occur when slicing a tensor.