ArcTensor

Type Alias ArcTensor 

pub type ArcTensor<T> = TensorBase<Arc<Vec<T>>, DynLayout>;
Reference-counted tensor with a dynamic dimension count.

This uses Arc<Vec<T>> rather than Arc<[T]> as the backing storage. This adds an extra indirection when accessing the data, but it enables cheap conversion between owned and reference-counted tensors.

Aliased Type

pub struct ArcTensor<T> { /* private fields */ }