Error types that are reported by various tensor operations.
Structs
- Dimension
Error - Error in a tensor operation if the dimension count is incorrect.
Enums
- Expand
Error - Errors that can occur while expanding a tensor.
- From
Data Error - Errors that can occur when constructing a tensor from existing data.
- Reshape
Error - Errors that can occur while reshaping a layout or tensor.
- Slice
Error - Errors that can occur when slicing a tensor.